import { includes } from 'lodash-es';
import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { DatabaseInfo, DatabaseStatus, SaveStateI, wsSelectState } from '../api/wsState';

/** Message format of the error message from the backend */
export type ErrorMessage = {
  errorStatus: any;
  queryID: any;
};

/** Cache type */
export type SessionCacheI = {
  currentSaveState?: string; // id of the current save state
  saveStates: Record<string, SaveStateI>;
};

// Define the initial state using that type
export const initialState: SessionCacheI = {
  currentSaveState: undefined,
  saveStates: {},
};

export const sessionSlice = createSlice({
  name: 'session',
  initialState,
  reducers: {
    updateCurrentSaveState(state, action: PayloadAction<string | undefined>) {
      state.currentSaveState = action.payload;
      wsSelectState(state.currentSaveState);
    },
    updateSaveStateList(state, action: PayloadAction<SaveStateI[]>) {
      state.saveStates = {};
      action.payload.forEach((ss) => {
        state.saveStates[ss.id] = ss;
      });

      if (!state.currentSaveState || !(state.currentSaveState in state.saveStates)) {
        if (Object.keys(state.saveStates).length > 0) {
          state.currentSaveState = Object.keys(state.saveStates)[0];
        } else state.currentSaveState = undefined;
      }
      wsSelectState(state.currentSaveState);
    },
    addSaveState(state, action: PayloadAction<SaveStateI>) {
      if (state.saveStates === undefined) state.saveStates = {};
      state.saveStates[action.payload.id] = action.payload;
      state.currentSaveState = action.payload.id;
      wsSelectState(state.currentSaveState);
    },
    testedSaveState(state, action: PayloadAction<string>) {
      if (action.payload in state.saveStates) {
        state.saveStates[action.payload].db.status = DatabaseStatus.tested;
      }
    },
  },
});

export const { updateCurrentSaveState, updateSaveStateList, addSaveState, testedSaveState } = sessionSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const sessionCacheState = (state: RootState) => state.sessionCache;

export default sessionSlice.reducer;
