import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { globalConfigSchemaTypes, localConfigSchemaType } from '../../vis/Types';

export enum Visualizations {
  NodeLink = 'NodeLinkVis',
  Paohvis = 'PaohVis',
  RawJSON = 'RawJSONVis',
  Table = 'TableVis',
}

type VisState = {
  activeVisualization: Visualizations;
  settings: {
    general: globalConfigSchemaTypes;
    [id: string]: localConfigSchemaType;
  };
};

export const initialState: VisState = {
  activeVisualization: Visualizations.NodeLink,
  settings: {
    general: {
      // Example general setting
      showLegend: {
        value: false,
        label: 'Show legend',
        type: 'boolean',
      },
    },
  },
};

export const visualizationSlice = createSlice({
  name: 'visualization',
  initialState,
  reducers: {
    addVisualization: (state, action: PayloadAction<{ id: string; settings: localConfigSchemaType }>) => {
      const { id, settings } = action.payload;
      state.settings[id] = settings;
    },
    removeVisualization: (state, action: PayloadAction<string>) => {
      if (state.settings[action.payload]) {
        delete state.settings[action.payload];
      }
    },
    updateVisualizationSettings: (state, action: PayloadAction<{ id: string; settings: any }>) => {
      const { id, settings } = action.payload;
      state.settings[id] = { ...state.settings[id], ...settings };
    },
    updateGeneralSettings: (state, action: PayloadAction<any>) => {
      state.settings.general = action.payload;
    },
    setActiveVisualization: (state, action: PayloadAction<Visualizations>) => {
      state.activeVisualization = action.payload;
    },
    updateSettings: (state, action: PayloadAction<any>) => {
      state.settings.visualization = action.payload;
    },
  },
});

export const {
  addVisualization,
  removeVisualization,
  updateVisualizationSettings,
  updateGeneralSettings,
  setActiveVisualization,
  updateSettings,
} = visualizationSlice.actions;

export const visualizationState = (state: RootState) => state.visualize;
export const visualizationSettings = (state: RootState) => ({
  general: state.visualize.settings.general,
  [state.visualize.activeVisualization]: state.visualize.settings[state.visualize.activeVisualization],
});

export default visualizationSlice.reducer;
