import { useEffect, useState } from 'react';
import { Dialog, DialogProps } from '../../components/Dialog';
import React from 'react';
import { useAppDispatch, useSchemaSettings } from '../../data-access';
import { SchemaSettings, setSchemaSettings } from '../../data-access/store/schemaSlice';
import { FormActions, FormBody, FormCard, FormControl, FormHBar, FormTitle, FormDiv } from '../../components/forms';
import { Layouts } from '../../graph-layout';
import Input from '../../components/inputs';

export const SchemaDialog = (props: DialogProps) => {
  const settings = useSchemaSettings();
  const dispatch = useAppDispatch();
  const [state, setState] = React.useState<SchemaSettings>(settings);
  const [display, setDisplay] = useState<string[]>([]);
  const [opacity, setOpacity] = useState<number>(0);

  useEffect(() => {
    setState(settings);
  }, [settings, props.open]);

  function submit() {
    dispatch(setSchemaSettings(state));
    props.onClose();
  }

  return (
    <>
      {props.open && (
        <FormDiv hAnchor="left">
          <FormCard>
            <FormBody
              onSubmit={(e) => {
                e.preventDefault();
                submit();
              }}
            >
              <FormTitle title="Quick Settings" onClose={props.onClose} />
              <FormHBar />
              <FormControl>
                <Input
                  type="checkbox"
                  value={display}
                  options={['Points', 'Line', 'Box']}
                  onChange={(value: string[]) => {
                    setDisplay(value);
                  }}
                />
              </FormControl>
              <FormHBar />
              <FormControl>
                <Input
                  type="slider"
                  label="Opacity"
                  unit={'%'}
                  value={opacity}
                  min={0}
                  max={100}
                  step={1}
                  onChange={(value: number) => setOpacity(value)}
                />
              </FormControl>
              <FormHBar />
              <FormControl>
                <label className="label">
                  <span className="label-text">Histogram</span>
                </label>
                ...
              </FormControl>
              <FormHBar />
              <FormControl>
                <Input
                  type="dropdown"
                  label="Type of Connection"
                  value={state.connectionType}
                  options={['Default', 'Step', 'Straight', 'Bezier']}
                  onChange={(value: string | number) => {
                    setState({ ...state, connectionType: value as any });
                  }}
                />
              </FormControl>
              <FormHBar />
              <FormControl>
                <Input
                  type="dropdown"
                  label="Layout Type"
                  value={state.layoutName}
                  options={Object.values(Layouts)}
                  onChange={(value: string | number) => {
                    setState({ ...state, layoutName: value as any });
                  }}
                />
              </FormControl>
              <FormHBar />

              <FormActions onClose={props.onClose} />
            </FormBody>
          </FormCard>
        </FormDiv>
      )}
    </>
  );
};
