import React from 'react';
import { useAppDispatch } from '@graphpolaris/shared/lib/data-access';
import { addVisualization } from '../data-access/store/visualizationSlice';
import {
  useGraphQueryResult,
  useML,
  useSchemaGraph,
  useVisualizationSettings,
  useVisualizationState,
} from '@graphpolaris/shared/lib/data-access/store/hooks';
import { localConfigPropTypes, globalConfigPropTypes, VISComponentType } from './Types';
import { NodeLinkComponent, PaohVisComponent, RawJSONComponent, TableComponent } from './visualizations';

// export * from './rawjsonvis';
// export * from './nodelink/nodelinkvis';
// export * from './paohvis/paohvis';
// export * from './semanticsubstrates/semanticsubstrates';
// export * from './table_vis/tableVis';
// export * from './mapvis/mapvis';

export const Visualizations = {
  TableVis: TableComponent,
  PaohVis: PaohVisComponent,
  RawJSONVis: RawJSONComponent,
  NodeLinkVis: NodeLinkComponent,
  MapVis: '',
};

export const createVisualizationComponent = () => {
  return () => {
    const dispatch = useAppDispatch();
    const vis = useVisualizationState();
    const settings = useVisualizationSettings();
    const graphQueryResult = useGraphQueryResult();
    const schema = useSchemaGraph();
    const ml = useML();

    const VisualizationComponent: VISComponentType = Visualizations[vis.activeVisualization];

    React.useEffect(() => {
      dispatch(addVisualization({ id: VisualizationComponent.displayName, settings: VisualizationComponent.localConfigSchema }));
    }, [vis.activeVisualization]);

    const globalConfig: globalConfigPropTypes =
      settings.general &&
      Object.keys(settings.general).reduce((propsObject, val) => {
        return {
          ...propsObject,
          [val]: vis.settings.general[val].value,
        };
      }, {});

    const displayName = vis.activeVisualization as keyof typeof Visualizations;

    const localConfig: localConfigPropTypes<typeof displayName> =
      settings[vis.activeVisualization] &&
      (Object.keys(settings[vis.activeVisualization]).reduce((propsObject, val) => {
        return {
          ...propsObject,
          [val]: vis.settings[vis.activeVisualization][val].value,
        };
      }, {}) as localConfigPropTypes<typeof displayName>);

    return (
      localConfig && (
        <div className="w-full h-full">
          <VisualizationComponent.VIS
            data={graphQueryResult}
            schema={schema}
            ml={ml}
            dispatch={dispatch}
            globalConfig={globalConfig}
            localConfig={localConfig}
          />
        </div>
      )
    );
  };
};
